@@ -20,6 +20,7 @@ from merlin.datasets.ecommerce.aliccp.dataset import (
     prepare_alliccp,
     transform_aliccp,
 )
+from merlin.datasets.ecommerce.booking.dataset import get_booking, transform_booking
 from merlin.datasets.ecommerce.dressipi.dataset import get_dressipi2022
 
 __all__ = [
@@ -28,4 +29,6 @@ __all__ = [
     "get_aliccp",
     "default_aliccp_transformation",
     "get_dressipi2022",
+    "get_booking",
+    "transform_booking",
 ]
