@@ -1,15 +1,33 @@
-from typing import Optional, Union
+#
+# Copyright (c) 2021, NVIDIA CORPORATION.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+import os
+from typing import Dict, Optional, Union
 
 import numpy as np
 import tensorflow as tf
 from packaging import version
 
 import merlin.io
+from merlin.models.io import save_merlin_metadata
 from merlin.models.tf.core import combinators
 from merlin.models.tf.core.prediction import TopKPrediction
 from merlin.models.tf.inputs.base import InputBlockV2
 from merlin.models.tf.inputs.embedding import CombinerType, EmbeddingTable
-from merlin.models.tf.models.base import BaseModel
+from merlin.models.tf.models.base import BaseModel, get_output_schema
 from merlin.models.tf.outputs.topk import TopKOutput
 from merlin.models.tf.utils import tf_utils
 from merlin.schema import ColumnSchema, Schema, Tags
@@ -139,7 +157,7 @@ class Encoder(tf.keras.Model):
 
         return merlin.io.Dataset(predictions)
 
-    def call(self, inputs, training=False, testing=False, targets=None):
+    def call(self, inputs, training=False, testing=False, targets=None, **kwargs):
         return combinators.call_sequentially(
             list(self.to_call),
             inputs=inputs,
@@ -147,6 +165,7 @@ class Encoder(tf.keras.Model):
             targets=targets,
             training=training,
             testing=testing,
+            **kwargs,
         )
 
     def build(self, input_shape):
@@ -189,6 +208,36 @@ class Encoder(tf.keras.Model):
             _arg_spec = self._saved_model_arg_spec
             self._saved_model_arg_spec = ([_arg_spec[0][0]], _arg_spec[1])
 
+    def save(
+        self,
+        export_path: Union[str, os.PathLike],
+        include_optimizer=True,
+        save_traces=True,
+    ) -> None:
+        """Saves the model to export_path as a Tensorflow Saved Model.
+        Along with merlin model metadata.
+
+        Parameters
+        ----------
+        export_path : Union[str, os.PathLike]
+            Path where model will be saved to
+        include_optimizer : bool, optional
+            If False, do not save the optimizer state, by default True
+        save_traces : bool, optional
+            When enabled, will store the function traces for each layer. This
+            can be disabled, so that only the configs of each layer are
+            stored, by default True
+        """
+        super().save(
+            export_path,
+            include_optimizer=include_optimizer,
+            save_traces=save_traces,
+            save_format="tf",
+        )
+        input_schema = self.schema
+        output_schema = get_output_schema(export_path)
+        save_merlin_metadata(export_path, self, input_schema, output_schema)
+
     @property
     def to_call(self):
         if self.pre:
@@ -275,6 +324,9 @@ class TopKEncoder(Encoder, BaseModel):
     post: Optional[tf.keras.layers.Layer]
         A block to use after getting the top-k prediction scores
         By default None
+    target: str, optional
+        The name of the target. This is required when multiple targets are provided.
+        By default None
     """
 
     def __init__(
@@ -286,12 +338,13 @@ class TopKEncoder(Encoder, BaseModel):
         k: int = 10,
         pre: Optional[tf.keras.layers.Layer] = None,
         post: Optional[tf.keras.layers.Layer] = None,
+        target: str = None,
         **kwargs,
     ):
         if isinstance(topk_layer, TopKOutput):
             topk_output = topk_layer
         else:
-            topk_output = TopKOutput(to_call=topk_layer, candidates=candidates, k=k, **kwargs)
+            topk_output = TopKOutput(to_call=topk_layer, candidates=candidates, k=k, target=target)
         self.k = k
 
         Encoder.__init__(self, query_encoder, topk_output, pre=pre, post=post, **kwargs)
@@ -340,6 +393,38 @@ class TopKEncoder(Encoder, BaseModel):
         )
         return cls(query_encoder, topk_output, **kwargs)
 
+    def compile(
+        self,
+        optimizer="rmsprop",
+        loss=None,
+        metrics=None,
+        loss_weights=None,
+        weighted_metrics=None,
+        run_eagerly=None,
+        steps_per_execution=None,
+        jit_compile=None,
+        k: int = None,
+        **kwargs,
+    ):
+        """Extend the compile method of `BaseModel` to set the threshold `k`
+        of the top-k encoder.
+        """
+        if k is not None:
+            self.topk_layer._k = k
+            self.k = k
+        BaseModel.compile(
+            self,
+            optimizer=optimizer,
+            loss=loss,
+            metrics=metrics,
+            weighted_metrics=weighted_metrics,
+            run_eagerly=run_eagerly,
+            loss_weights=loss_weights,
+            steps_per_execution=steps_per_execution,
+            jit_compile=jit_compile,
+            **kwargs,
+        )
+
     @property
     def topk_layer(self):
         return self.blocks[-1].to_call
@@ -451,6 +536,8 @@ class EmbeddingEncoder(Encoder):
         name=None,
         dtype=None,
         dynamic=False,
+        post: Optional[tf.keras.layers.Layer] = None,
+        embeddings_l2_batch_regularization: Optional[Union[float, Dict[str, float]]] = 0.0,
     ):
         if isinstance(schema, ColumnSchema):
             col = schema
@@ -472,9 +559,10 @@ class EmbeddingEncoder(Encoder):
             name=name,
             dtype=dtype,
             dynamic=dynamic,
+            l2_batch_regularization_factor=embeddings_l2_batch_regularization,
         )
 
-        super().__init__(table, tf.keras.layers.Lambda(lambda x: x[col_name]))
+        super().__init__(table, tf.keras.layers.Lambda(lambda x: x[col_name]), post=post)
 
     def to_dataset(self, gpu=None) -> merlin.io.Dataset:
         return self.blocks[0].to_dataset(gpu=gpu)
