@@ -198,6 +198,9 @@ class EmbeddingTable(EmbeddingTableBase):
        for example, or generally for any layer that manipulates tensors
        using Python control flow. If `False`, we assume that the layer can
        safely be used to generate a static computation graph.
+    l2_batch_regularization_factor: float, optional
+        Factor for L2 regularization of the embeddings vectors (from the current batch only)
+        by default 0.0
     **kwargs: Forwarded Keras Layer parameters
     """
 
@@ -217,6 +220,7 @@ class EmbeddingTable(EmbeddingTableBase):
         dtype=None,
         dynamic=False,
         table=None,
+        l2_batch_regularization_factor=0.0,
         **kwargs,
     ):
         """Create an EmbeddingTable."""
@@ -249,6 +253,7 @@ class EmbeddingTable(EmbeddingTableBase):
             )
         self.sequence_combiner = sequence_combiner
         self.supports_masking = True
+        self.l2_batch_regularization_factor = l2_batch_regularization_factor
 
     def select_by_tag(self, tags: Union[Tags, Sequence[Tags]]) -> Optional["EmbeddingTable"]:
         """Select features in EmbeddingTable by tags.
@@ -387,13 +392,17 @@ class EmbeddingTable(EmbeddingTableBase):
             inputs = list_col_to_ragged(inputs)
 
         # Eliminating the last dim==1 of dense tensors before embedding lookup
-        if isinstance(inputs, tf.Tensor):
+        if isinstance(inputs, tf.Tensor) or (
+            isinstance(inputs, tf.RaggedTensor) and inputs.shape[-1] == 1
+        ):
             inputs = tf.squeeze(inputs, axis=-1)
 
         if isinstance(inputs, (tf.RaggedTensor, tf.SparseTensor)):
             if self.sequence_combiner and isinstance(self.sequence_combiner, str):
                 if isinstance(inputs, tf.RaggedTensor):
                     inputs = inputs.to_sparse()
+                if len(inputs.dense_shape) == 3 and inputs.dense_shape[-1] == 1:
+                    inputs = tf.sparse.reshape(inputs, inputs.dense_shape[:-1])
                 out = tf.nn.safe_embedding_lookup_sparse(
                     self.table.embeddings, inputs, None, combiner=self.sequence_combiner
                 )
@@ -410,6 +419,9 @@ class EmbeddingTable(EmbeddingTableBase):
         else:
             out = call_layer(self.table, inputs, **kwargs)
 
+        if self.l2_batch_regularization_factor > 0:
+            self.add_loss(self.l2_batch_regularization_factor * tf.reduce_sum(tf.square(out)))
+
         if self._dtype_policy.compute_dtype != self._dtype_policy.variable_dtype:
             # Instead of casting the variable as in most layers, cast the output, as
             # this is mathematically equivalent but is faster.
@@ -440,7 +452,10 @@ class EmbeddingTable(EmbeddingTableBase):
 
         first_dims = input_shape
         if (self.sequence_combiner is not None) or (input_shape.rank > 1 and input_shape[-1] == 1):
-            first_dims = input_shape[:-1]
+            if len(input_shape) == 3:
+                first_dims = [input_shape[0]]
+            else:
+                first_dims = input_shape[:-1]
         output_shapes = tf.TensorShape(first_dims + [self.dim])
 
         return output_shapes
@@ -483,6 +498,7 @@ def Embeddings(
     post: Optional[BlockType] = None,
     aggregation: Optional[TabularAggregationType] = None,
     block_name: str = "embeddings",
+    l2_batch_regularization_factor: Optional[Union[float, Dict[str, float]]] = 0.0,
     **kwargs,
 ) -> ParallelBlock:
     """Creates a ParallelBlock with an EmbeddingTable for each categorical feature
@@ -526,7 +542,10 @@ def Embeddings(
         Transformation block to apply for aggregating the inputs, by default None
     block_name: str, optional
         Name of the block, by default "embeddings"
-
+    l2_batch_regularization_factor: Optional[float, Dict[str, float]] = 0.0
+        Factor for L2 regularization of the embeddings vectors (from the current batch only)
+        If a dictionary is provided, the keys are feature names and the values are
+        regularization factors
     Returns
     -------
     ParallelBlock
@@ -542,6 +561,8 @@ def Embeddings(
         kwargs["activity_regularizer"] = activity_regularizer
     if sequence_combiner:
         kwargs["sequence_combiner"] = sequence_combiner
+    if l2_batch_regularization_factor:
+        kwargs["l2_batch_regularization_factor"] = l2_batch_regularization_factor
 
     tables = {}
 
@@ -552,7 +573,10 @@ def Embeddings(
             tables[table_name].add_feature(col)
         else:
             tables[table_name] = table_cls(
-                _get_dim(col, dim, infer_dim_fn), col, name=table_name, **table_kwargs
+                _get_dim(col, dim, infer_dim_fn),
+                col,
+                name=table_name,
+                **table_kwargs,
             )
 
     return ParallelBlock(
@@ -870,6 +894,9 @@ class EmbeddingFeatures(TabularBlock):
         table: TableConfig = self.feature_config[name].table
         table_var = self.embedding_tables[table.name].embeddings
         if isinstance(val, tf.SparseTensor):
+            if len(val.dense_shape) == 3 and val.dense_shape[-1] == 1:
+                val = tf.sparse.reshape(val, val.dense_shape[:-1])
+
             out = tf.nn.safe_embedding_lookup_sparse(table_var, val, None, combiner=table.combiner)
         else:
             if output_sequence:
@@ -1013,24 +1040,19 @@ class EmbeddingFeatures(TabularBlock):
             feature_configs[key] = feature_config_dict
 
         config["feature_config"] = feature_configs
+        config["l2_reg"] = self.l2_reg
 
         return config
 
     @classmethod
     def from_config(cls, config):
         # Deserialize feature_config
-        feature_configs, table_configs = {}, {}
+        feature_configs = {}
         for key, val in config["feature_config"].items():
-            feature_params = deepcopy(val)
-            table_params = feature_params["table"]
-            if "name" in table_configs:
-                feature_params["table"] = table_configs["name"]
-            else:
-                table = deserialize_table_config(table_params)
-                if table.name:
-                    table_configs[table.name] = table
-                feature_params["table"] = table
-            feature_configs[key] = FeatureConfig(**feature_params)
+            table = deserialize_table_config(val["table"])
+            feature_config_params = {**val, "table": table}
+            feature_configs[key] = FeatureConfig(**feature_config_params)
+
         config["feature_config"] = feature_configs
 
         # Set `add_default_pre to False` since pre will be provided from the config
