@@ -6,7 +6,7 @@ import tensorflow as tf
 
 from merlin.core.dispatch import DataFrameType, concat_columns, get_lib
 from merlin.models.tf.core.base import Block
-from merlin.models.tf.dataset import BatchedDataset
+from merlin.models.tf.loader import Loader
 from merlin.models.tf.models.base import Model, RetrievalModel
 from merlin.models.utils.schema_utils import select_targets
 from merlin.schema import Schema, Tags
@@ -154,6 +154,12 @@ def model_encode(model, batch):
 
     model_outputs = model(batch[0])
 
+    # handle when the model outputs a NamedTuple
+    if hasattr(model_outputs, "to_df"):
+        return model_outputs.to_df()
+    elif hasattr(model_outputs, "_asdict"):
+        model_outputs = model_outputs._asdict()
+
     if isinstance(model_outputs, dict):
         return get_lib().DataFrame({key: encode_output(val) for key, val in model_outputs.items()})
 
@@ -175,7 +181,7 @@ def data_iterator_func(schema, batch_size: int = 512):
     targets = select_targets(schema).column_names
 
     def data_iterator(dataset):
-        return BatchedDataset(
+        return Loader(
             merlin.io.dataset.Dataset(dataset),
             batch_size=batch_size,
             cat_names=cat_cols,
