@@ -19,18 +19,23 @@ from typing import Any, Callable, Dict, Optional, Text, Union
 
 import torch
 
-from merlin.models.utils.doc_utils import docstring_parameter
-from merlin.schema import Schema, Tags
-
-from ...utils.schema import categorical_cardinalities, get_embedding_sizes_from_schema
-from ..tabular.base import (
+from merlin.models.torch.features.base import InputBlock
+from merlin.models.torch.tabular.base import (
     TABULAR_MODULE_PARAMS_DOCSTRING,
     FilterFeatures,
     TabularAggregationType,
     TabularTransformationType,
 )
-from ..utils.torch_utils import calculate_batch_size_from_input_size, get_output_sizes_from_schema
-from .base import InputBlock
+from merlin.models.torch.utils.torch_utils import (
+    calculate_batch_size_from_input_size,
+    get_output_sizes_from_schema,
+)
+from merlin.models.utils.doc_utils import docstring_parameter
+from merlin.models.utils.schema_utils import (
+    categorical_cardinalities,
+    get_embedding_sizes_from_schema,
+)
+from merlin.schema import Schema, Tags
 
 EMBEDDING_FEATURES_PARAMS_DOCSTRING = """
     feature_config: Dict[str, FeatureConfig]
@@ -153,7 +158,7 @@ class EmbeddingFeatures(InputBlock):
         Optional[EmbeddingFeatures]
             Returns the ``EmbeddingFeatures`` for the dataset schema
         """
-        # TODO: propogate item-id from ITEM_ID tag
+        # TODO: propagate item-id from ITEM_ID tag
 
         if tags:
             schema = schema.select_by_tag(tags)
@@ -336,7 +341,7 @@ class SoftEmbeddingFeatures(EmbeddingFeatures):
         Optional[SoftEmbeddingFeatures]
             Returns a ``SoftEmbeddingFeatures`` instance from the dataset schema
         """
-        # TODO: propogate item-id from ITEM_ID tag
+        # TODO: propagate item-id from ITEM_ID tag
 
         if tags:
             schema = schema.select_by_tag(tags)
