@@ -6,11 +6,11 @@ from tests.conftest import REPO_ROOT
 
 
 @testbook(REPO_ROOT / "examples/03-Exploring-different-models.ipynb", execute=False)
-def test_example_03_exploring_different_models(tb):
+def test_example_03_exploring_different_models(tb, tmpdir):
     tb.inject(
-        """
+        f"""
         import os
-        os.environ["DATA_FOLDER"] = "/tmp/data/"
+        os.environ["DATA_FOLDER"] = "{tmpdir}"
         os.environ["NUM_ROWS"] = "999"
         """
     )
@@ -21,6 +21,7 @@ def test_example_03_exploring_different_models(tb):
         [
             "auc",
             "loss",
+            "loss_batch",
             "regularization_loss",
         ]
     )
@@ -29,6 +30,7 @@ def test_example_03_exploring_different_models(tb):
         [
             "auc_1",
             "loss",
+            "loss_batch",
             "regularization_loss",
         ]
     )
@@ -37,6 +39,7 @@ def test_example_03_exploring_different_models(tb):
         [
             "auc_2",
             "loss",
+            "loss_batch",
             "regularization_loss",
         ]
     )
@@ -45,6 +48,7 @@ def test_example_03_exploring_different_models(tb):
         [
             "auc_3",
             "loss",
+            "loss_batch",
             "regularization_loss",
         ]
     )
@@ -53,8 +57,9 @@ def test_example_03_exploring_different_models(tb):
         [
             "auc_4",
             "loss",
+            "loss_batch",
             "regularization_loss",
         ]
     )
-    assert os.path.isfile("results.txt")
+    assert os.path.isfile(os.path.join(tmpdir, "results.txt"))
     tb.execute_cell(NUM_OF_CELLS - 2)
