@@ -6,14 +6,14 @@ from tests.conftest import REPO_ROOT
 
 
 @testbook(REPO_ROOT / "examples/04-Exporting-ranking-models.ipynb", execute=False)
-def test_example_04_exporting_ranking_models(tb):
+def test_example_04_exporting_ranking_models(tb, tmpdir):
     tb.inject(
-        """
+        f"""
         import os
-        os.environ["DATA_FOLDER"] = "/tmp/data/"
+        os.environ["DATA_FOLDER"] = "{tmpdir}"
         os.environ["NUM_ROWS"] = "999"
         """
     )
     tb.execute()
-    assert os.path.isdir("dlrm")
-    assert os.path.isdir("workflow")
+    assert os.path.isdir(os.path.join(tmpdir, "dlrm"))
+    assert os.path.isdir(os.path.join(tmpdir, "workflow"))
