@@ -8,11 +8,13 @@ from tests.conftest import REPO_ROOT
 @testbook(
     REPO_ROOT / "examples/06-Define-your-own-architecture-with-Merlin-Models.ipynb", execute=False
 )
-def test_example_06_defining_own_architecture(tb):
+def test_example_06_defining_own_architecture(tb, tmpdir):
     tb.inject(
-        """
+        f"""
+        import os
         from unittest.mock import patch
         from merlin.datasets.synthetic import generate_data
+        os.environ["DATA_FOLDER"] = "{tmpdir}"
         mock_train, mock_valid = generate_data(
             input="movielens-1m",
             num_rows=1000,
@@ -27,5 +29,5 @@ def test_example_06_defining_own_architecture(tb):
     )
     tb.execute()
     metrics = tb.ref("metrics")
-    assert set(metrics.keys()) == set(["auc", "loss", "regularization_loss"])
-    assert os.path.isdir("custom_dlrm")
+    assert set(metrics.keys()) == set(["auc", "loss", "loss_batch", "regularization_loss"])
+    assert os.path.isdir(os.path.join(tmpdir, "custom_dlrm"))
