@@ -8,5 +8,5 @@ from tests.conftest import REPO_ROOT
 )
 def test_usecase_pretrained_embeddings(tb):
     tb.execute()
-    model = tb.ref("model")
-    assert set(model.history.history.keys()) == set(["auc", "loss", "regularization_loss"])
+    history = tb.ref("history")
+    assert set(history.keys()) == set(["auc", "loss", "loss_batch", "regularization_loss"])
